<?php

declare(strict_types=1);

namespace App\Controller;

use App\Entity\DrinkType;
use App\Service\DrinkTypeService;
use App\Service\InventoryService;
use InvalidArgumentException;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Views\Twig;

class DrinkTypeController
{
    public function __construct(
        private readonly DrinkTypeService $drinkTypeService,
        private readonly InventoryService $inventoryService,
        private readonly Twig $view
    ) {}

    public function index(Response $response): Response
    {
        $drinkTypes = $this->drinkTypeService->getAllDrinkTypes();

        // Get current stock levels for all drink types
        $drinkTypesWithStock = [];
        foreach ($drinkTypes as $drinkType) {
            $currentStock = $this->inventoryService->getCurrentStockLevel($drinkType);
            $drinkTypesWithStock[] = [
                'id' => $drinkType->getId(),
                'name' => $drinkType->getName(),
                'description' => $drinkType->getDescription(),
                'desiredStock' => $drinkType->getDesiredStock(),
                'currentStock' => $currentStock,
                'createdAt' => $drinkType->getCreatedAt(),
                'updatedAt' => $drinkType->getUpdatedAt(),
            ];
        }

        return $this->view->render($response, 'drink-types/index.twig', [
            'drinkTypes' => $drinkTypesWithStock,
        ]);
    }

    public function show(Response $response, int $id): Response
    {
        $drinkType = $this->drinkTypeService->getDrinkTypeById($id);

        if (!$drinkType instanceof DrinkType) {
            // Redirect to drink types list with error message
            // In a real application, you might want to use flash messages
            return $response->withHeader('Location', '/drink-types')->withStatus(302);
        }

        // Get current stock information for the drink type
        $currentStock = $this->inventoryService->getCurrentStockLevel($drinkType);

        // Get inventory records for the drink type
        $inventoryRecords = $this->inventoryService->getInventoryRecordsByDrinkType($drinkType);

        return $this->view->render($response, 'drink-types/show.twig', [
            'drinkType' => $drinkType,
            'currentStock' => $currentStock,
            'inventoryRecords' => $inventoryRecords,
        ]);
    }

    public function create(Response $response): Response
    {
        return $this->view->render($response, 'drink-types/create.twig');
    }

    public function store(Request $request, Response $response): Response
    {
        $data = $request->getParsedBody();

        $name = $data['name'] ?? '';
        $description = $data['description'] ?? null;
        $desiredStock = isset($data['desired_stock']) ? (int) $data['desired_stock'] : 0;

        try {
            $this->drinkTypeService->createDrinkType($name, $description, $desiredStock);

            // Redirect to drink types list
            return $response->withHeader('Location', '/drink-types')->withStatus(302);
        } catch (InvalidArgumentException $e) {
            // Render the form again with error message
            return $this->view->render($response, 'drink-types/create.twig', [
                'error' => $e->getMessage(),
                'data' => $data,
            ]);
        }
    }

    public function edit(Request $request, Response $response, int $id): Response
    {
        $drinkType = $this->drinkTypeService->getDrinkTypeById($id);

        if (!$drinkType instanceof DrinkType) {
            // Redirect to drink types list
            return $response->withHeader('Location', '/drink-types')->withStatus(302);
        }

        return $this->view->render($response, 'drink-types/edit.twig', [
            'drinkType' => $drinkType,
        ]);
    }

    public function update(Request $request, Response $response, int $id): Response
    {
        $drinkType = $this->drinkTypeService->getDrinkTypeById($id);

        if (!$drinkType instanceof DrinkType) {
            // Redirect to drink types list
            return $response->withHeader('Location', '/drink-types')->withStatus(302);
        }

        $data = $request->getParsedBody();

        $name = $data['name'] ?? null;
        $description = $data['description'] ?? null;
        $desiredStock = isset($data['desired_stock']) ? (int) $data['desired_stock'] : null;

        try {
            $this->drinkTypeService->updateDrinkType($drinkType, $name, $description, $desiredStock);

            // Redirect to drink type details
            return $response->withHeader('Location', '/drink-types/' . $id)->withStatus(302);
        } catch (InvalidArgumentException $e) {
            // Render the form again with error message
            return $this->view->render($response, 'drink-types/edit.twig', [
                'drinkType' => $drinkType,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function delete(Request $request, Response $response, int $id): Response
    {
        $drinkType = $this->drinkTypeService->getDrinkTypeById($id);

        if ($drinkType instanceof DrinkType) {
            $this->drinkTypeService->deleteDrinkType($drinkType);
        }

        // Redirect to drink types list
        return $response->withHeader('Location', '/drink-types')->withStatus(302);
    }
}
