<?php

declare(strict_types=1);

namespace App\Controller;

use App\Entity\DrinkType;
use App\Service\DrinkTypeService;
use App\Service\InventoryService;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Views\Twig;

class InventoryController
{
    public function __construct(
        private readonly InventoryService $inventoryService,
        private readonly DrinkTypeService $drinkTypeService,
        private readonly Twig $view
    ) {}

    public function index(Response $response): Response
    {
        $stockLevels = $this->inventoryService->getAllDrinkTypesWithStockLevels();

        return $this->view->render($response, 'inventory/index.twig', [
            'stockLevels' => $stockLevels,
        ]);
    }

    public function showUpdateForm(Response $response): Response
    {
        $drinkTypes = $this->drinkTypeService->getAllDrinkTypes();

        return $this->view->render($response, 'inventory/update.twig', [
            'drinkTypes' => $drinkTypes,
        ]);
    }

    public function update(Request $request, Response $response): Response
    {
        $data = $request->getParsedBody();

        $drinkTypeId = isset($data['drink_type_id']) ? (int) $data['drink_type_id'] : 0;
        $quantity = isset($data['quantity']) ? (int) $data['quantity'] : 0;

        $drinkType = $this->drinkTypeService->getDrinkTypeById($drinkTypeId);

        if (!$drinkType instanceof DrinkType) {
            // Redirect to inventory page with error message
            return $response->withHeader('Location', '/inventory')->withStatus(302);
        }

        $this->inventoryService->updateStockLevel($drinkType, $quantity);

        // Redirect to inventory page
        return $response->withHeader('Location', '/inventory')->withStatus(302);
    }

    public function history(Response $response, ?int $id = null): Response
    {
        $drinkTypeId = $id;

        if ($drinkTypeId !== null && $drinkTypeId !== 0) {
            $drinkType = $this->drinkTypeService->getDrinkTypeById($drinkTypeId);

            if (!$drinkType instanceof DrinkType) {
                // Redirect to inventory page
                return $response->withHeader('Location', '/inventory')->withStatus(302);
            }

            $inventoryRecords = $this->inventoryService->getInventoryRecordsByDrinkType($drinkType);

            return $this->view->render($response, 'inventory/history.twig', [
                'drinkType' => $drinkType,
                'inventoryRecords' => $inventoryRecords,
            ]);
        }

        // If no drink type ID is provided, show history for all drink types
        $inventoryRecords = $this->inventoryService->getAllInventoryRecords();

        return $this->view->render($response, 'inventory/history.twig', [
            'inventoryRecords' => $inventoryRecords,
        ]);
    }

    public function lowStock(Response $response): Response
    {
        $lowStockItems = $this->inventoryService->getLowStockDrinkTypes();

        return $this->view->render($response, 'inventory/low-stock.twig', [
            'lowStockItems' => $lowStockItems,
        ]);
    }
}
