<?php

declare(strict_types=1);

namespace App\Controller;

use App\Entity\DrinkType;
use App\Entity\Order;
use App\Service\DrinkTypeService;
use App\Service\OrderService;
use InvalidArgumentException;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Views\Twig;

class OrderController
{
    public function __construct(
        private readonly OrderService $orderService,
        private readonly DrinkTypeService $drinkTypeService,
        private readonly Twig $view
    ) {}

    public function index(Response $response): Response
    {
        $orders = $this->orderService->getAllOrders();

        return $this->view->render($response, 'orders/index.twig', [
            'orders' => $orders,
        ]);
    }

    public function show(Response $response, int $id): Response
    {
        $order = $this->orderService->getOrderById($id);

        if (!$order instanceof Order) {
            // Redirect to orders list
            return $response->withHeader('Location', '/orders')->withStatus(302);
        }

        return $this->view->render($response, 'orders/show.twig', [
            'order' => $order,
        ]);
    }

    public function create(Response $response): Response
    {
        $drinkTypes = $this->drinkTypeService->getAllDrinkTypes();

        return $this->view->render($response, 'orders/create.twig', [
            'drinkTypes' => $drinkTypes,
        ]);
    }

    public function store(Request $request, Response $response): Response
    {
        $data = $request->getParsedBody();

        $items = [];

        // Process form data to create order items
        foreach ($data['items'] ?? [] as $item) {
            if (!empty($item['drink_type_id']) && !empty($item['quantity'])) {
                $items[] = [
                    'drinkTypeId' => (int) $item['drink_type_id'],
                    'quantity' => (int) $item['quantity'],
                ];
            }
        }

        try {
            $order = $this->orderService->createOrder($items);

            // Redirect to order details
            return $response->withHeader('Location', '/orders/' . $order->getId())->withStatus(302);
        } catch (InvalidArgumentException $e) {
            // Render the form again with error message
            return $this->view->render($response, 'orders/create.twig', [
                'error' => $e->getMessage(),
                'drinkTypes' => $this->drinkTypeService->getAllDrinkTypes(),
                'data' => $data,
            ]);
        }
    }

    public function createFromStock(Response $response): Response
    {
        try {
            $order = $this->orderService->createOrderFromStockLevels();

            // Redirect to order details
            return $response->withHeader('Location', '/orders/' . $order->getId())->withStatus(302);
        } catch (InvalidArgumentException) {
            // Redirect to orders list with error message
            return $response->withHeader('Location', '/orders')->withStatus(302);
        }
    }

    public function updateStatus(Request $request, Response $response, int $id): Response
    {
        $order = $this->orderService->getOrderById($id);

        if (!$order instanceof Order) {
            // Redirect to orders list
            return $response->withHeader('Location', '/orders')->withStatus(302);
        }

        $data = $request->getParsedBody();
        $status = $data['status'] ?? '';

        try {
            $this->orderService->updateOrderStatus($order, $status);

            // Redirect to order details
            return $response->withHeader('Location', '/orders/' . $id)->withStatus(302);
        } catch (InvalidArgumentException $e) {
            // Render the order details with error message
            return $this->view->render($response, 'orders/show.twig', [
                'order' => $order,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function addItem(Request $request, Response $response, int $id): Response
    {
        $order = $this->orderService->getOrderById($id);

        if (!$order instanceof Order) {
            // Redirect to orders list
            return $response->withHeader('Location', '/orders')->withStatus(302);
        }

        $data = $request->getParsedBody();
        $drinkTypeId = isset($data['drink_type_id']) ? (int) $data['drink_type_id'] : 0;
        $quantity = isset($data['quantity']) ? (int) $data['quantity'] : 0;

        $drinkType = $this->drinkTypeService->getDrinkTypeById($drinkTypeId);

        if (!$drinkType instanceof DrinkType) {
            // Render the order details with error message
            return $this->view->render($response, 'orders/show.twig', [
                'order' => $order,
                'error' => 'Invalid drink type',
            ]);
        }

        try {
            $this->orderService->addOrderItem($order, $drinkType, $quantity);

            // Redirect to order details
            return $response->withHeader('Location', '/orders/' . $id)->withStatus(302);
        } catch (InvalidArgumentException $e) {
            // Render the order details with error message
            return $this->view->render($response, 'orders/show.twig', [
                'order' => $order,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function removeItem(Response $response, int $id, int $itemId): Response
    {
        $orderId = $id;

        $order = $this->orderService->getOrderById($orderId);

        if (!$order instanceof Order) {
            // Redirect to orders list
            return $response->withHeader('Location', '/orders')->withStatus(302);
        }

        $orderItem = null;
        foreach ($order->getOrderItems() as $item) {
            if ($item->getId() === $itemId) {
                $orderItem = $item;
                break;
            }
        }

        if (!$orderItem) {
            // Redirect to order details
            return $response->withHeader('Location', '/orders/' . $orderId)->withStatus(302);
        }

        try {
            $this->orderService->removeOrderItem($order, $orderItem);

            // Redirect to order details
            return $response->withHeader('Location', '/orders/' . $orderId)->withStatus(302);
        } catch (InvalidArgumentException $e) {
            // Render the order details with error message
            return $this->view->render($response, 'orders/show.twig', [
                'order' => $order,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function delete(Response $response, int $id): Response
    {
        $order = $this->orderService->getOrderById($id);

        if (!$order instanceof Order) {
            // Redirect to orders list
            return $response->withHeader('Location', '/orders')->withStatus(302);
        }

        try {
            $this->orderService->deleteOrder($order);

            // Redirect to orders list
            return $response->withHeader('Location', '/orders')->withStatus(302);
        } catch (InvalidArgumentException $e) {
            // Render the order details with error message
            return $this->view->render($response, 'orders/show.twig', [
                'order' => $order,
                'error' => $e->getMessage(),
            ]);
        }
    }
}
