<?php

declare(strict_types=1);

namespace App\Controller;

use App\Entity\SystemConfig;
use App\Service\ConfigurationService;
use InvalidArgumentException;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Views\Twig;

class SettingsController
{
    public function __construct(
        private readonly ConfigurationService $configurationService,
        private readonly Twig $view
    ) {}

    public function index(Response $response): Response
    {

        $settings = [];
        foreach($this->configurationService->getAllConfigs() as $setting) {
            $settings[$setting->getKey()] = $setting->getValue();
        }
        return $this->view->render($response, 'settings/index.twig', [
            'configs' => $settings,
        ]);
    }

    public function update(Request $request, Response $response): Response
    {
        $data = $request->getParsedBody();

        // Process each configuration value from the form
        foreach ($data['configs'] ?? [] as $key => $value) {
            try {
                // Get existing config or create a new one
                $config = $this->configurationService->getConfigByKey($key);

                if ($config instanceof SystemConfig) {
                    $this->configurationService->updateConfig($config, null, $value);
                } else {
                    $this->configurationService->createConfig($key, $value);
                }
            } catch (InvalidArgumentException $e) {
                // If there's an error, render the form again with error message
                return $this->view->render($response, 'settings/index.twig', [
                    'configs' => $this->configurationService->getAllConfigs(),
                    'error' => $e->getMessage(),
                ]);
            }
        }

        // Redirect back to settings page with success message
        return $response->withHeader('Location', '/settings')->withStatus(302);
    }

    public function resetToDefaults(Response $response): Response
    {
        // Delete all existing configurations
        foreach ($this->configurationService->getAllConfigs() as $config) {
            $this->configurationService->deleteConfig($config);
        }

        // Initialize default configurations
        $this->configurationService->initializeDefaultConfigs();

        // Redirect back to settings page
        return $response->withHeader('Location', '/settings')->withStatus(302);
    }
}
