<?php

declare(strict_types=1);

namespace App\Middleware;

use Monolog\Logger;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Slim\Exception\HttpException;
use Slim\Psr7\Response;
use Slim\Views\Twig;
use Throwable;

class ErrorHandlerMiddleware implements MiddlewareInterface
{
    public function __construct(private readonly Logger $logger, private readonly Twig $twig, private readonly bool $displayErrorDetails = false) {}

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        try {
            return $handler->handle($request);
        } catch (HttpException $e) {
            return $this->handleException($e, $request, $e->getCode());
        } catch (Throwable $e) {
            return $this->handleException($e, $request);
        }
    }

    private function handleException(Throwable $e, ServerRequestInterface $request, int $statusCode = 500): ResponseInterface
    {
        $this->logger->error($e->getMessage(), [
            'exception' => $e,
            'request' => [
                'method' => $request->getMethod(),
                'uri' => (string) $request->getUri(),
            ],
        ]);

        $response = new Response($statusCode);

        // Only show error details in development
        $error = [
            'message' => $this->displayErrorDetails ? $e->getMessage() : 'An error occurred',
        ];

        if ($this->displayErrorDetails) {
            $error['trace'] = $e->getTraceAsString();
            $error['file'] = $e->getFile();
            $error['line'] = $e->getLine();
        }

        // Try to render an error page, fall back to JSON if Twig fails
        try {
            return $this->twig->render($response, 'error.twig', [
                'error' => $error,
                'status_code' => $statusCode,
            ]);
        } catch (Throwable $e) {
            $this->logger->error('Failed to render error template', ['exception' => $e]);

            $response = $response->withHeader('Content-Type', 'application/json');
            $response->getBody()->write(json_encode(['error' => $error]));

            return $response;
        }
    }
}
