<?php

declare(strict_types=1);

namespace App\Service;

use App\Entity\DrinkType;
use App\Entity\InventoryRecord;
use App\Repository\DrinkTypeRepository;
use App\Repository\InventoryRecordRepository;
use App\Service\ConfigurationService;
use App\Enum\SystemSettingKey;
use DateTimeImmutable;

class InventoryService
{
    public function __construct(
        private readonly InventoryRecordRepository $inventoryRecordRepository,
        private readonly DrinkTypeRepository $drinkTypeRepository,
        private readonly ConfigurationService $configService
    ) {}

    /**
     * Get all inventory records
     *
     * @return InventoryRecord[]
     */
    public function getAllInventoryRecords(): array
    {
        return $this->inventoryRecordRepository->findAll();
    }

    /**
     * Get inventory records for a specific drink type
     *
     * @param DrinkType $drinkType
     * @return InventoryRecord[]
     */
    public function getInventoryRecordsByDrinkType(DrinkType $drinkType): array
    {
        return $this->inventoryRecordRepository->findByDrinkType($drinkType);
    }

    /**
     * Get the latest inventory record for a specific drink type
     *
     * @param DrinkType $drinkType
     * @return InventoryRecord|null
     */
    public function getLatestInventoryRecord(DrinkType $drinkType): ?InventoryRecord
    {
        return $this->inventoryRecordRepository->findLatestByDrinkType($drinkType);
    }

    /**
     * Get inventory records within a specific time range
     *
     * @param DateTimeImmutable $start
     * @param DateTimeImmutable $end
     * @return InventoryRecord[]
     */
    public function getInventoryRecordsByTimeRange(DateTimeImmutable $start, DateTimeImmutable $end): array
    {
        return $this->inventoryRecordRepository->findByTimestampRange($start, $end);
    }

    /**
     * Get the current stock level for a specific drink type
     *
     * @param DrinkType $drinkType
     * @return int
     */
    public function getCurrentStockLevel(DrinkType $drinkType): int
    {
        $latestRecord = $this->getLatestInventoryRecord($drinkType);
        return $latestRecord instanceof InventoryRecord ? $latestRecord->getQuantity() : 0;
    }

    /**
     * Get the difference between current and desired stock for a specific drink type
     *
     * @param DrinkType $drinkType
     * @return int Positive value means excess stock, negative value means shortage
     */
    public function getStockDifference(DrinkType $drinkType): int
    {
        $currentStock = $this->getCurrentStockLevel($drinkType);
        $desiredStock = $drinkType->getDesiredStock();

        return $currentStock - $desiredStock;
    }

    /**
     * Update the stock level for a specific drink type
     *
     * @param DrinkType $drinkType
     * @param int $quantity
     * @param DateTimeImmutable|null $timestamp
     * @return InventoryRecord
     */
    public function updateStockLevel(
        DrinkType $drinkType,
        int $quantity,
        ?DateTimeImmutable $timestamp = null
    ): InventoryRecord {
        $inventoryRecord = new InventoryRecord(
            $drinkType,
            $quantity,
            $timestamp
        );

        $this->inventoryRecordRepository->save($inventoryRecord);

        return $inventoryRecord;
    }

    /**
     * Get all drink types with their current stock levels
     *
     * @return array<int, array{drinkType: DrinkType, currentStock: int, desiredStock: int, difference: int}>
     */
    public function getAllDrinkTypesWithStockLevels(): array
    {
        $drinkTypes = $this->drinkTypeRepository->findAll();
        $result = [];

        foreach ($drinkTypes as $drinkType) {
            $currentStock = $this->getCurrentStockLevel($drinkType);
            $desiredStock = $drinkType->getDesiredStock();
            $difference = $currentStock - $desiredStock;

            $result[] = [
                'drinkType' => $drinkType,
                'currentStock' => $currentStock,
                'desiredStock' => $desiredStock,
                'difference' => $difference,
            ];
        }

        return $result;
    }

    /**
     * Get all drink types with low stock (current stock below low stock threshold)
     *
     * @return array<int, array{drinkType: DrinkType, currentStock: int, desiredStock: int, shortage: int}>
     */
    public function getLowStockDrinkTypes(): array
    {
        $lowStockThreshold = (float) $this->configService->getConfigValue(SystemSettingKey::LOW_STOCK_THRESHOLD->value, '50') / 100;
        $drinkTypes = $this->drinkTypeRepository->findAll();
        $result = [];

        foreach ($drinkTypes as $drinkType) {
            $currentStock = $this->getCurrentStockLevel($drinkType);
            $desiredStock = $drinkType->getDesiredStock();

            // Check if current stock is below the threshold percentage of desired stock
            if ($desiredStock > 0 && $currentStock < ($desiredStock * $lowStockThreshold)) {
                $shortage = $desiredStock - $currentStock;

                $result[] = [
                    'drinkType' => $drinkType,
                    'currentStock' => $currentStock,
                    'desiredStock' => $desiredStock,
                    'shortage' => $shortage,
                ];
            }
        }

        return $result;
    }

    /**
     * Get all drink types with critical stock (current stock below critical stock threshold)
     *
     * @return array<int, array{drinkType: DrinkType, currentStock: int, desiredStock: int, shortage: int}>
     */
    public function getCriticalStockDrinkTypes(): array
    {
        $criticalStockThreshold = (float) $this->configService->getConfigValue(SystemSettingKey::CRITICAL_STOCK_THRESHOLD->value, '25') / 100;
        $drinkTypes = $this->drinkTypeRepository->findAll();
        $result = [];

        foreach ($drinkTypes as $drinkType) {
            $currentStock = $this->getCurrentStockLevel($drinkType);
            $desiredStock = $drinkType->getDesiredStock();

            // Check if current stock is below the threshold percentage of desired stock
            if ($desiredStock > 0 && $currentStock < ($desiredStock * $criticalStockThreshold)) {
                $shortage = $desiredStock - $currentStock;

                $result[] = [
                    'drinkType' => $drinkType,
                    'currentStock' => $currentStock,
                    'desiredStock' => $desiredStock,
                    'shortage' => $shortage,
                ];
            }
        }

        return $result;
    }
}
