<?php

declare(strict_types=1);

namespace App\Service;

use InvalidArgumentException;
use App\Entity\DrinkType;
use App\Entity\Order;
use App\Entity\OrderItem;
use App\Repository\DrinkTypeRepository;
use App\Repository\OrderItemRepository;
use App\Repository\OrderRepository;
use DateTimeImmutable;

class OrderService
{
    public function __construct(
        private readonly OrderRepository $orderRepository,
        private readonly OrderItemRepository $orderItemRepository,
        private readonly DrinkTypeRepository $drinkTypeRepository,
        private readonly InventoryService $inventoryService
    ) {}

    /**
     * Get all orders
     *
     * @return Order[]
     */
    public function getAllOrders(): array
    {
        return $this->orderRepository->findAll();
    }

    /**
     * Get an order by ID
     *
     * @param int $id
     * @return Order|null
     */
    public function getOrderById(int $id): ?Order
    {
        return $this->orderRepository->find($id);
    }

    /**
     * Get orders by status
     *
     * @param string $status
     * @return Order[]
     */
    public function getOrdersByStatus(string $status): array
    {
        return $this->orderRepository->findByStatus($status);
    }

    /**
     * Get orders within a specific date range
     *
     * @param DateTimeImmutable $start
     * @param DateTimeImmutable $end
     * @return Order[]
     */
    public function getOrdersByDateRange(DateTimeImmutable $start, DateTimeImmutable $end): array
    {
        return $this->orderRepository->findByDateRange($start, $end);
    }

    /**
     * Create a new order
     *
     * @param array<int, array{drinkTypeId: int, quantity: int}> $items
     * @return Order
     * @throws InvalidArgumentException If a drink type ID is invalid
     */
    public function createOrder(array $items): Order
    {
        $order = new Order();
        $this->orderRepository->save($order);

        foreach ($items as $item) {
            $drinkType = $this->drinkTypeRepository->find($item['drinkTypeId']);
            if ($drinkType === null) {
                throw new InvalidArgumentException("Invalid drink type ID: {$item['drinkTypeId']}");
            }

            $orderItem = new OrderItem($drinkType, $item['quantity'], $order);
            $this->orderItemRepository->save($orderItem);
        }

        return $order;
    }

    /**
     * Create an order based on current stock levels
     *
     * @return Order
     */
    public function createOrderFromStockLevels(): Order
    {
        $lowStockItems = $this->inventoryService->getAllDrinkTypesWithStockLevels();
        $orderItems = [];

        foreach ($lowStockItems as $item) {
            if ($item['currentStock'] < $item['desiredStock']) {
                $orderItems[] = [
                    'drinkTypeId' => $item['drinkType']->getId(),
                    'quantity' => $item['desiredStock'] - $item['currentStock'],
                ];
            }
        }

        return $this->createOrder($orderItems);
    }

    /**
     * Update an order's status
     *
     * @param Order $order
     * @param string $status
     * @return Order
     * @throws InvalidArgumentException If the status is invalid
     */
    public function updateOrderStatus(Order $order, string $status): Order
    {
        $order->setStatus($status);
        $this->orderRepository->save($order);

        // If the order is fulfilled, update the inventory
        if ($status === Order::STATUS_FULFILLED) {
            $this->updateInventoryFromOrder($order);
        }

        return $order;
    }

    /**
     * Update inventory based on a fulfilled order
     *
     * @param Order $order
     * @return void
     */
    private function updateInventoryFromOrder(Order $order): void
    {
        foreach ($order->getOrderItems() as $orderItem) {
            $drinkType = $orderItem->getDrinkType();
            $currentStock = $this->inventoryService->getCurrentStockLevel($drinkType);
            $newStock = $currentStock + $orderItem->getQuantity();

            $this->inventoryService->updateStockLevel($drinkType, $newStock);
        }
    }

    /**
     * Add an item to an order
     *
     * @param Order $order
     * @param DrinkType $drinkType
     * @param int $quantity
     * @return OrderItem
     * @throws InvalidArgumentException If the order is not in 'new' or 'in_work' status
     */
    public function addOrderItem(Order $order, DrinkType $drinkType, int $quantity): OrderItem
    {
        if (!in_array($order->getStatus(), [Order::STATUS_NEW, Order::STATUS_IN_WORK], true)) {
            throw new InvalidArgumentException("Cannot add items to an order with status '{$order->getStatus()}'");
        }

        // Check if the order already has an item for this drink type
        $existingItem = $this->orderItemRepository->findByOrderAndDrinkType($order, $drinkType);

        if ($existingItem instanceof OrderItem) {
            // Update the existing item
            $existingItem->setQuantity($existingItem->getQuantity() + $quantity);
            $this->orderItemRepository->save($existingItem);
            return $existingItem;
        }
        // Create a new item
        $orderItem = new OrderItem($drinkType, $quantity, $order);
        $this->orderItemRepository->save($orderItem);
        return $orderItem;
    }

    /**
     * Remove an item from an order
     *
     * @param Order $order
     * @param OrderItem $orderItem
     * @return void
     * @throws InvalidArgumentException If the order is not in 'new' or 'in_work' status
     */
    public function removeOrderItem(Order $order, OrderItem $orderItem): void
    {
        if (!in_array($order->getStatus(), [Order::STATUS_NEW, Order::STATUS_IN_WORK], true)) {
            throw new InvalidArgumentException("Cannot remove items from an order with status '{$order->getStatus()}'");
        }

        $order->removeOrderItem($orderItem);
        $this->orderItemRepository->remove($orderItem);
    }

    /**
     * Delete an order
     *
     * @param Order $order
     * @return void
     * @throws InvalidArgumentException If the order is not in 'new' status
     */
    public function deleteOrder(Order $order): void
    {
        if ($order->getStatus() !== Order::STATUS_NEW) {
            throw new InvalidArgumentException("Cannot delete an order with status '{$order->getStatus()}'");
        }

        $this->orderRepository->remove($order);
    }
}
