<?php

declare(strict_types=1);

namespace App\Service;

use App\Entity\InventoryRecord;
use App\Entity\DrinkType;
use App\Repository\DrinkTypeRepository;
use App\Repository\InventoryRecordRepository;
use App\Repository\OrderRepository;
use DateTimeImmutable;

class StockAdjustmentService
{
    private const int DEFAULT_LOOKBACK_ORDERS = 5;
    private const float DEFAULT_ADJUSTMENT_MAGNITUDE = 0.2; // 20%
    private const float DEFAULT_ADJUSTMENT_THRESHOLD = 0.1; // 10%

    public function __construct(
        private readonly DrinkTypeRepository $drinkTypeRepository,
        private readonly InventoryRecordRepository $inventoryRecordRepository,
        private readonly OrderRepository $orderRepository,
        private readonly ConfigurationService $configService,
        private readonly InventoryService $inventoryService
    ) {}

    /**
     * Analyze consumption patterns and adjust desired stock levels
     *
     * @return array<int, array{drinkType: DrinkType, oldDesiredStock: int, newDesiredStock: int, adjustmentReason: string}>
     */
    public function adjustStockLevels(): array
    {
        $lookbackOrders = (int) $this->configService->getConfigValue(
            'stock_adjustment_lookback_orders',
            (string) self::DEFAULT_LOOKBACK_ORDERS
        );

        $adjustmentMagnitude = (float) $this->configService->getConfigValue(
            'stock_adjustment_magnitude',
            (string) self::DEFAULT_ADJUSTMENT_MAGNITUDE
        );

        $adjustmentThreshold = (float) $this->configService->getConfigValue(
            'stock_adjustment_threshold',
            (string) self::DEFAULT_ADJUSTMENT_THRESHOLD
        );

        $drinkTypes = $this->drinkTypeRepository->findAll();
        $result = [];

        foreach ($drinkTypes as $drinkType) {
            $consumptionRate = $this->calculateConsumptionRateFromOrders($drinkType, $lookbackOrders);

            if ($consumptionRate === null) {
                continue; // Skip if we don't have enough data
            }

            $currentDesiredStock = $drinkType->getDesiredStock();
            // For the suggested stock, we use the consumption rate directly as it's already weighted
            $suggestedStock = (int) ceil($consumptionRate);

            // Only adjust if the difference is significant
            $difference = abs($suggestedStock - $currentDesiredStock) / max(1, $currentDesiredStock);

            if ($difference >= $adjustmentThreshold) {
                // Calculate the new desired stock with a gradual adjustment
                $adjustment = (int) ceil(($suggestedStock - $currentDesiredStock) * $adjustmentMagnitude);
                $newDesiredStock = $currentDesiredStock + $adjustment;

                // Ensure we don't go below 1
                $newDesiredStock = max(1, $newDesiredStock);

                // Update the drink type
                $drinkType->setDesiredStock($newDesiredStock);
                $this->drinkTypeRepository->save($drinkType);

                $result[] = [
                    'drinkType' => $drinkType,
                    'oldDesiredStock' => $currentDesiredStock,
                    'newDesiredStock' => $newDesiredStock,
                    'adjustmentReason' => $suggestedStock > $currentDesiredStock
                        ? 'Increased consumption rate'
                        : 'Decreased consumption rate',
                ];
            }
        }

        return $result;
    }

    /**
     * Calculate the weighted consumption rate based on the last N orders for a drink type
     *
     * @param DrinkType $drinkType
     * @param int $lookbackOrders Number of past orders to consider
     * @return float|null Weighted consumption rate, or null if not enough data
     */
    private function calculateConsumptionRateFromOrders(
        DrinkType $drinkType,
        int $lookbackOrders
    ): ?float {
        // Get the last N orders for this drink type
        $orders = $this->orderRepository->findLastOrdersForDrinkType($drinkType, $lookbackOrders);

        // Need at least one order to calculate consumption
        if (count($orders) === 0) {
            return null;
        }

        // Calculate weighted consumption
        $totalWeightedConsumption = 0;
        $totalWeight = 0;

        // Define weights - most recent order has highest weight
        // For example, with 5 orders: weights are 5, 4, 3, 2, 1
        $weights = range($lookbackOrders, 1);

        foreach ($orders as $index => $order) {
            // Get the order items for this drink type
            $orderItems = $order->getOrderItems();
            $quantity = 0;

            // Sum up quantities for this drink type
            foreach ($orderItems as $item) {
                if ($item->getDrinkType()->getId() === $drinkType->getId()) {
                    $quantity += $item->getQuantity();
                }
            }

            // Apply weight to this order's consumption
            $weight = $weights[$index] ?? 1; // Fallback to weight 1 if index is out of bounds
            $totalWeightedConsumption += $quantity * $weight;
            $totalWeight += $weight;
        }

        // If total weight is 0, return null (shouldn't happen, but just in case)
        if ($totalWeight === 0) {
            return null;
        }

        // Calculate weighted average consumption
        return $totalWeightedConsumption / $totalWeight;
    }

    /**
     * Calculate the average daily consumption rate for a drink type
     *
     * @deprecated Use calculateConsumptionRateFromOrders instead
     * @param DrinkType $drinkType
     * @param DateTimeImmutable $startDate
     * @param DateTimeImmutable $endDate
     * @return float|null Average daily consumption rate, or null if not enough data
     */
    private function calculateConsumptionRate(
        DrinkType $drinkType,
        DateTimeImmutable $startDate,
        DateTimeImmutable $endDate
    ): ?float {
        // Get inventory records for the period
        $inventoryRecords = $this->inventoryRecordRepository->findByDrinkType($drinkType);

        // Filter records within the date range
        $inventoryRecords = array_filter($inventoryRecords, function ($record) use ($startDate, $endDate): bool {
            $timestamp = $record->getTimestamp();
            return $timestamp >= $startDate && $timestamp <= $endDate;
        });

        // Sort by timestamp
        usort($inventoryRecords, fn($a, $b): int => $a->getTimestamp() <=> $b->getTimestamp());

        // Need at least two records to calculate consumption
        if (count($inventoryRecords) < 2) {
            return null;
        }

        // Calculate total consumption
        $totalConsumption = 0;
        $previousRecord = null;

        foreach ($inventoryRecords as $record) {
            if ($previousRecord instanceof InventoryRecord) {
                $previousQuantity = $previousRecord->getQuantity();
                $currentQuantity = $record->getQuantity();
                $daysDifference = $record->getTimestamp()->diff($previousRecord->getTimestamp())->days;

                // If days difference is 0, use 1 to avoid division by zero
                $daysDifference = max(1, $daysDifference);

                // If current quantity is less than previous, consumption occurred
                if ($currentQuantity < $previousQuantity) {
                    $consumption = $previousQuantity - $currentQuantity;
                    $totalConsumption += $consumption;
                }
            }

            $previousRecord = $record;
        }

        // Calculate days between first and last record
        $firstRecord = reset($inventoryRecords);
        $lastRecord = end($inventoryRecords);
        $totalDays = $lastRecord->getTimestamp()->diff($firstRecord->getTimestamp())->days;

        // If total days is 0, use 1 to avoid division by zero
        $totalDays = max(1, $totalDays);

        // Calculate average daily consumption
        return $totalConsumption / $totalDays;
    }

    /**
     * Get the consumption history for a drink type
     *
     * @param DrinkType $drinkType
     * @param DateTimeImmutable $startDate
     * @param DateTimeImmutable $endDate
     * @return array<int, array{date: DateTimeImmutable, consumption: int}>
     */
    public function getConsumptionHistory(
        DrinkType $drinkType,
        DateTimeImmutable $startDate,
        DateTimeImmutable $endDate
    ): array {
        // Get inventory records for the period
        $inventoryRecords = $this->inventoryRecordRepository->findByDrinkType($drinkType);

        // Filter records within the date range
        $inventoryRecords = array_filter($inventoryRecords, function ($record) use ($startDate, $endDate): bool {
            $timestamp = $record->getTimestamp();
            return $timestamp >= $startDate && $timestamp <= $endDate;
        });

        // Sort by timestamp
        usort($inventoryRecords, fn($a, $b): int => $a->getTimestamp() <=> $b->getTimestamp());

        // Calculate consumption between each record
        $consumptionHistory = [];
        $previousRecord = null;

        foreach ($inventoryRecords as $record) {
            if ($previousRecord !== null) {
                $previousQuantity = $previousRecord->getQuantity();
                $currentQuantity = $record->getQuantity();

                // If current quantity is less than previous, consumption occurred
                if ($currentQuantity < $previousQuantity) {
                    $consumption = $previousQuantity - $currentQuantity;
                    $consumptionHistory[] = [
                        'date' => $record->getTimestamp(),
                        'consumption' => $consumption,
                    ];
                }
            }

            $previousRecord = $record;
        }

        return $consumptionHistory;
    }
}
